<?php
/**
 * Init
 *
 * @author Semenov Alexander <semenov@skeeks.com>
 * @link http://skeeks.com/
 * @copyright 2010-2014 SkeekS (Sx)
 * @date 19.02.2015
 * @since 1.0.0
 */
include_once __DIR__ . '/Helper.php';
include_once __DIR__ . '/Object.php';

/**
 * Class Base
 */
class Base extends Object
{
    public $params          = [];
    public $setWritable     = [];
    public $setExecutable   = [];
    public $action          = "help";


    public function init()
    {
        $this->_ensure();
        $this->_initParams();
    }


    /**
     * @return $this
     */
    public function actionHelp()
    {
        $this->stdoutN('Possible actions:');
        $this->stdoutN(' ');
        $methods = get_class_methods($this->className());

        if ($methods)
        {
            $actions = [];
            foreach ($methods as $methodName)
            {
                if (substr($methodName, 0, 6) == 'action')
                {
                    $actions[] = lcfirst(substr($methodName, 6, strlen($methodName)));
                }
            }

            $this->stdoutN(implode(" | ", $actions));
        }

        return $this;
    }

    protected function _ensure()
    {
        if (!extension_loaded('mcrypt'))
        {
            throw new Exception('The mcrypt PHP extension is required by SkeekS Cms Application.');
        }
    }

    /**
     * @return $this
     */
    protected function _initParams()
    {
        $this->params = Helper::getParams();
        if (isset($this->params[0]))
        {
            $this->action = $this->params[0];
            unset($this->params[0]);
        }
        return $this;
    }


    /**
     * @param $actionName
     * @return $this
     * @throws Exception
     */
    public function runAction($actionName)
    {
        if (!isset($this->params["no" . ucfirst($actionName)]))
        {
            $actionName = "action" . ucfirst($actionName);

            if (!method_exists($this, $actionName))
            {
                throw new Exception("Action '{$actionName}' not found");
            }

            $this->{$actionName}();
        }

        return $this;
    }

    /**
     *
     */
    public function run()
    {
        if ($this->existLockFile())
        {
            if (isset($this->params['unlock']))
            {
                $this->deleteLockFile();
            } else
            {
                $this->stdoutN('This action is currently locked. You can run it with the option --unlock');
                die;
            }
        }

        $this->createLockFile();

        try
        {
            $this->_run();
        } catch (Exception $e)
        {
            $this->stdoutN($e->getMessage());
            $this->stdoutN("");
            $this->actionHelp();
        }

        $this->deleteLockFile();

        return $this;
    }

    protected function _run()
    {
        if ($actionName = $this->action)
        {
            $actionName = "action" . ucfirst($actionName);

            if (!method_exists($this, $actionName))
            {
                throw new Exception("Action '{$actionName}' not found");
            }

            $this->{$actionName}();
        }

        return $this;
    }



    public function createLockFile()
    {
        if (!is_dir(ROOT_DIR . '/install/runtime'))
        {
            mkdir(ROOT_DIR . '/install/runtime', 0777, true);
        }

        if (!is_writable(ROOT_DIR . '/install/runtime'))
        {
            Helper::setWritable(ROOT_DIR, ['install/runtime']);
        }

        $file = fopen($this->getLockFilePath(), "w");
        fwrite($file, time());
        fclose($file);
    }

    /**
     * @return bool
     */
    public function existLockFile()
    {
        return (bool) file_exists($this->getLockFilePath());
    }

    /**
     *
     */
    public function deleteLockFile()
    {
        if ($this->existLockFile())
        {
            unlink($this->getLockFilePath());
        }
    }

    /**
     * @return string
     */
    public function getLockFilePath()
    {
        return ROOT_DIR . '/install/runtime/' . $this->action . '.lock';
    }


    public function systemCmdRoot($cmd)
    {
        $this->stdoutN(' - system cmd: ' . $cmd);
        system("pwd " . ROOT_DIR . "; " . $cmd);
        return $this;
    }

    /**
     * @param $cmd
     * @return $this
     */
    public function systemCmd($cmd)
    {
        $this->stdoutN(' - system cmd: ' . $cmd);
        system($cmd);
        return $this;
    }

    /**
     * @param $text
     * @return $this
     */
    public function stdoutN($text = '')
    {
        $this->stdout("{$text}" . PHP_EOL);
        return $this;
    }

    /**
     * @param string $text
     * @return $this
     */
    public function stdoutBlock($text = '')
    {
        $this->stdoutN('');
        $this->stdout(" ****** {$text} *****" . PHP_EOL);
        $this->stdoutN('');
        return $this;
    }



    /**
     * Gets input from STDIN and returns a string right-trimmed for EOLs.
     *
     * @param boolean $raw If set to true, returns the raw string without trimming
     * @return string the string read from stdin
     */
    public function stdin($raw = false)
    {
        return $raw ? fgets(\STDIN) : rtrim(fgets(\STDIN), PHP_EOL);
    }

    /**
     * Prints a string to STDOUT.
     *
     * @param string $string the string to print
     * @return int|boolean Number of bytes printed or false on error
     */
    public function stdout($string)
    {
        return fwrite(\STDOUT, $string);
    }

    /**
     * Prints a string to STDERR.
     *
     * @param string $string the string to print
     * @return int|boolean Number of bytes printed or false on error
     */
    public function stderr($string)
    {
        return fwrite(\STDERR, $string);
    }
}