<?php
/**
 * Helper
 *
 * @author Semenov Alexander <semenov@skeeks.com>
 * @link http://skeeks.com/
 * @copyright 2010-2014 SkeekS (Sx)
 * @date 19.02.2015
 * @since 1.0.0
 */
class Helper
{
    function getFileList($root, $basePath = '')
    {
        $files = [];
        $handle = opendir($root);
        while (($path = readdir($handle)) !== false) {
            if ($path === '.svn' || $path === '.' || $path === '..') {
                continue;
            }
            $fullPath = "$root/$path";
            $relativePath = $basePath === '' ? $path : "$basePath/$path";
            if (is_dir($fullPath)) {
                $files = array_merge($files, getFileList($fullPath, $relativePath));
            } else {
                $files[] = $relativePath;
            }
        }
        closedir($handle);
        return $files;
    }

    function copyFile($root, $source, $target, &$all, $params)
    {
        if (!is_file($root . '/' . $source)) {
            echo "       skip $target ($source not exist)\n";
            return true;
        }
        if (is_file($root . '/' . $target)) {
            if (file_get_contents($root . '/' . $source) === file_get_contents($root . '/' . $target)) {
                echo "  unchanged $target\n";
                return true;
            }
            if ($all) {
                echo "  overwrite $target\n";
            } else {
                echo "      exist $target\n";
                echo "            ...overwrite? [Yes|No|All|Quit] ";


                $answer = !empty($params['overwrite']) ? $params['overwrite'] : trim(fgets(STDIN));
                if (!strncasecmp($answer, 'q', 1)) {
                    return false;
                } else {
                    if (!strncasecmp($answer, 'y', 1)) {
                        echo "  overwrite $target\n";
                    } else {
                        if (!strncasecmp($answer, 'a', 1)) {
                            echo "  overwrite $target\n";
                            $all = true;
                        } else {
                            echo "       skip $target\n";
                            return true;
                        }
                    }
                }
            }
            file_put_contents($root . '/' . $target, file_get_contents($root . '/' . $source));
            return true;
        }
        echo "   generate $target\n";
        @mkdir(dirname($root . '/' . $target), 0777, true);
        file_put_contents($root . '/' . $target, file_get_contents($root . '/' . $source));
        return true;
    }

    static public function getParams()
    {
        $rawParams = [];
        if (isset($_SERVER['argv'])) {
            $rawParams = $_SERVER['argv'];
            array_shift($rawParams);
        }

        $params = [];
        foreach ($rawParams as $param) {
            if (preg_match('/^--(\w+)(=(.*))?$/', $param, $matches)) {
                $name = $matches[1];
                $params[$name] = isset($matches[3]) ? $matches[3] : true;
            } else {
                $params[] = $param;
            }
        }
        return $params;
    }

    static public function setWritable($root, $paths)
    {
        foreach ($paths as $writable) {
            echo "      chmod 0777 $writable\n";
            @chmod("$root/$writable", 0777);
        }
    }

    static public function setExecutable($root, $paths)
    {
        foreach ($paths as $executable) {
            echo "      chmod 0755 $executable\n";
            @chmod("$root/$executable", 0755);
        }
    }

    function setCookieValidationKey($root, $paths)
    {
        foreach ($paths as $file) {
            echo "   generate cookie validation key in $file\n";
            $file = $root . '/' . $file;
            $length = 32;
            $bytes = mcrypt_create_iv($length, MCRYPT_DEV_URANDOM);
            $key = strtr(substr(base64_encode($bytes), 0, $length), '+/=', '_-.');
            $content = preg_replace('/(("|\')cookieValidationKey("|\')\s*=>\s*)(""|\'\')/', "\\1'$key'", file_get_contents($file));
            file_put_contents($file, $content);
        }
    }

    function createSymlink($links)
    {
        foreach ($links as $link => $target) {
            echo "    symlink $target as $link\n";
            if (!is_link($link)) {
                symlink($target, $link);
            }
        }
    }

}