<?php
/**
 * Abstract
 *
 * @author Semenov Alexander <semenov@skeeks.com>
 * @link http://skeeks.com/
 * @copyright 2010-2014 SkeekS (Sx)
 * @date 12.10.2014
 * @since 1.0.0
 */



/**
 * Class Cx_Application
 */
abstract class Cx_Application
    extends Cx_Entity
{

    /**
     * @var Cx_Controller_Request
     */
    public $request     = null;

    /**
     * @var Cx_Controller_Response|Cx_Controller_Response|null
     */
    public $response    = null;


    /**
     * @param $name
     * @throws Cx_Exception
     */
    public function getTemplateEngine($name)
    {
        $className = 'Cx_View_TemplateEngine_' . ucfirst($name);
        if (!class_exists($className))
        {
            throw new Cx_Exception("Ну нйден шаблонизатор" . $name);
        }

        $engine = new $className();
        if (!is_subclass_of($engine, 'Cx_View_TemplateEngine'))
        {
            throw new Cx_Exception("Должен быть Cx_View_TemplateEngine");
        }

        return $engine;
    }

    /**
     *
     */
    public function init()
    {
        //$this->_initErrorHandlers();
    }


    /**
     * @return Cx_Controller_Request
     */
    public function getRequest()
    {
        if ($this->request === null)
        {
            $this->request = new Cx_Controller_Request();
        }

        return $this->request;
    }

    /**
     * @return Cx_Controller_Response|null
     */
    public function getResponse()
    {
        if ($this->response === null)
        {
            $this->response = $this->getRequest()->createDefaultResponse();
        }

        return $this->response;
    }


    /**
     * @abstract
     * @return bool
     */
    abstract public function isCli();


    /**
     * Runs the application
     *
     * @return mixed
     */
    public function run()
    {
        $result = $this->processRequest();
        return $result;
    }


    abstract public function processRequest();


    protected $_urlManager = null;
    /**
     * @return Cx_UrlManager
     */
    public function getUrlManager()
    {
        if ($this->_urlManager === null)
        {
            $this->_urlManager = new Cx_UrlManager();
        }

        return $this->_urlManager;
    }


    /**
     * @param Cx_Controller_Request $request
     * @param Cx_Controller_Response $response
     * @return $this|null
     * @throws Cx_Exception
     */
    public function dispatch(Cx_Controller_Request $request = null, Cx_Controller_Response $response = null)
    {
        $this->_preDispatch($request, $response);
        $this->_dispatch($request, $response);
        $this->_postDispatch($request, $response);

        return $this;
    }

    protected function _dispatch(Cx_Controller_Request $request = null, Cx_Controller_Response $response = null)
    {
        if ($request === null)
        {
            $request = $this->getRequest();
        }

        if ($response === null)
        {
            $response = $this->getResponse();
        }

        if (!$request->isRouted())
        {
            throw new Cx_Exception("Couldn't dispatch a non-routed request.");
        }

        $ac = $this->createActionController($request);
        $ac->dispatch($request, $response);

        if (!$request->isDispatched())
        {
            //request wasn't dispatched
            //restart the dispatch process
            //@todo: write here the code
            //@start redispatch
            throw new Cx_Exception("Request wasn't dispatched.");
        }

        return $this;
    }

    protected function _preDispatch(Cx_Controller_Request $request = null, Cx_Controller_Response $response = null)
    {}
    protected function _postDispatch(Cx_Controller_Request $request = null, Cx_Controller_Response $response = null)
    {}


    /**
     * @param Cx_Controller_Request $request
     * @return mixed
     * @throws Cx_Exception
     */
    static protected function createActionController(Cx_Controller_Request $request)
    {

        $controllerClass = "\\Cx_Controller_" . ucfirst($request->getControllerName());

        if (!class_exists($controllerClass))
        {
            throw new Cx_Exception("Controller class '{$controllerClass}' wasn't found.");
        }

        if (!is_subclass_of($controllerClass, 'Cx_Controller_Action'))
        {
            throw new Cx_Exception("Controller class '{$controllerClass}' should inherit "
                . " from 'Cx_Controller_Action'."
            );
        }

        //create the controller
        return new $controllerClass(strtolower($request->getControllerName()));
    }


}


