<?php
/**
 * Array
 *
 * @date 15.05.14
 * @copyright skeeks.com
 * @author Semenov Alexander <semenov@skeeks.com>
 */
class Cx_Array
{
    protected $_array = [];

    /**
     * @param string $delimetr
     * @param $string
     * @param null|int $limit
     * @return static
     */
    static public function createFromExplode($delimetr = ":", $string)
    {
        return new static((array) explode($delimetr, $string));
    }

    public function __construct($array = [])
    {
        $this->_array = $array;
    }

    /**
     * @param $callback
     * @return $this
     * @throws Cx_ExceptionSxInvalidArgument
     */
    public function eachCallback($callback)
    {
        if (!is_callable($callback))
        {
            throw new Cx_ExceptionSxInvalidArgument("callback must be function");
        }

        $newArray = [];

        foreach ($this->_array as $key => $value)
        {
            $newArray[$key] = $callback($value);
        }

        $this->_array = $newArray;
        return $this;
    }

    /**
     * @return int
     */
    public function count()
    {
        return count($this->_array);
    }

    /**
     * @return bool
     */
    public function isEmpty()
    {
        return (bool) !$this->_array;
    }

    /**
     * @return $this
     */
    public function removeEmptyElements()
    {
        $result = [];

        foreach($this->_array as $key => $value)
        {
            if (!empty($value))
            {
                $result[$key] = $value;
            }
        }

        $this->_array = $result;

        return $this;
    }


    /**
     * @return mixed|null
     */
    public function getLast()
    {
        if (!$this->isEmpty())
        {
            $array = $this->_array;
            return end($array);
        }

        return null;
    }


    /**
     * @return mixed|null
     */
    public function getFirst()
    {
        $array = $this->_array;
        if ($array)
        {
            return array_shift($array);
        }

        return null;
    }

    /**
     * @return array
     */
    public function toArray()
    {
        return $this->_array;
    }


    /**
     * @param array $array
     * @param null $key
     * @param null $default
     * @return mixed|null
     */
    static public function getKeyOrArray(array $array, $key = null, $default = null)
    {
        if (null === $key)
        {
            return $array;
        }

        return array_key_exists($key, $array) ? $array[$key] : $default;
    }


    /**
     * @param array $entities
     * @param bool $smart
     * @return array
     */
    static public function entitiesToArray($entities = [], $smart = false)
    {
        $result = array();

        if ($entities)
        {
            foreach ($entities as $key => $entity)
            {
                if ($entity instanceof \Cx_Entity)
                {
                    $result[$key] = ($smart === true) ? $entity->toArraySmart() : $entity->toArray();
                } else if (is_array($entity))
                {
                    $result[$key] = $entity;
                }
            }
        }

        return $result;
    }




    /**
     * @param array $array
     * @param string $key
     * @param string $value
     * @param array $first
     * @return array
     */
    static public function multiOptionsFromArray($array = [], $key = "id", $value = "name", $first = [])
    {
        $result = array();

        $result = $first;

        if ($found = $array)
        {
            foreach ($found as $num => $data)
            {
                if (isset($data[$key]) && isset($data[$value]))
                {
                    $result[$data[$key]] = $data[$value];
                }
            }
        }

        return $result;
    }
    /**
     *
     * Построить мультиопции
     *
     * @param Zend_Db_Select $select
     * @param string $key
     * @param string $value
     * @param array $first
     * @return array
     */
    static public function multiOptionsFromSelect(Zend_Db_Select $select, $key = "id", $value = "name", $first = [])
    {
        $result = array();

        $result = $first;

        if ($found = $select->query()->fetchAll())
        {
            foreach ($found as $num => $data)
            {
                if (isset($data[$key]) && isset($data[$value]))
                {
                    $result[$data[$key]] = $data[$value];
                }
            }
        }

        return $result;
    }


    /**
     * @param array $array
     * @return array
     */
    static public function onlyNotEmpty(array $array)
    {
        $result = array();

        foreach($array as $key => $value)
        {
            if (!empty($value))
            {
                $result[$key] = $value;
            }
        }

        return $result;
    }
}