<?php
/**
 * Default class autoloader
 *
 * @author Semenov Alexander <semenov@skeeks.com>
 * @link http://skeeks.com/
 * @copyright 2010-2014 SkeekS (Sx)
 * @date 11.10.2014
 * @since 1.0.0
 */

/**
 * Interface Ix_ClassAutoloader
 */
interface Ix_ClassAutoloader
{
    /**
     * @abstract
     * @return Ix_ClassAutoloader
     */
    public function register();

    /**
     * @abstract
     * @return bool
     */
    public function isRegistered();

    /**
     * @abstract
     * @param  string $className
     * @return bool
     */
    public function loadClass($className);
}

/**
 * Class Cx_ClassAutoloader
 */
class Cx_ClassAutoloader
    implements Ix_ClassAutoloader
{
    /**
     * @var bool
     */
    protected $_isRegistered = false;

    /**
     * @var string
     */
    protected $_codePaths = array();

    /**
     * @param array $codePaths
     */
    public function __construct($codePaths)
    {
        if (is_array($codePaths))
        {
            $this->_codePaths = $codePaths;
        } else
        {
            $this->_codePaths = array($codePaths);
        }
    }

    /**
     * @return Cx_ClassAutoloader
     */
    public function register()
    {
        if (!$this->_isRegistered)
        {
            spl_autoload_register(array($this, 'loadClass'));
            $this->_isRegistered = true;
        }

        return $this;
    }

    /**
     * @return bool
     */
    public function isRegistered()
    {
        return $this->_isRegistered;
    }

    /**
     * @param  string $className
     * @return bool
     * @throws Cx_Exception
     */
    public function loadClass($className)
    {
        //echo $className . PHP_EOL;
        /**
         * Название класса должно начинаться с Cx_ или \Cx_
         */
        if (preg_match('/(^|\\\)(i|t|c)x\_\w+$/i', $className))
        {

            if ($this->_codePaths)
            {
                $classNamespaces    = explode("\\", $className);
                $classNameReal      = end($classNamespaces);
                $countParts         = count($classNamespaces);

                if ($countParts == 1)
                {
                    $filePath = DIRECTORY_SEPARATOR . SX_CODE_DIR_NAME . DIRECTORY_SEPARATOR . str_replace('_', DIRECTORY_SEPARATOR, substr($className, 3)) . '.php';
                }

                foreach ($this->_codePaths as $codePath)
                {
                    $filePaths[] = $codePath . $filePath;
                }

            }
            
            return $this->_requireAndCheck($filePaths, $className);
        }

        return false;
    }

    /**
     * @param array $filePaths
     * @param $className
     * @return bool
     * @throws Cx_Exception
     */
    protected function _requireAndCheck(array $filePaths, $className)
    {
        $filePathReal = null;

        if ($filePaths)
        {
            foreach ($filePaths as $filePath)
            {
                if (file_exists($filePath))
                {
                    $filePathReal = $filePath;
                    break;
                }
            }
        }
        if (null === $filePathReal)
        {
            return false;
            //throw new Cx_Exception("File '{$filePath}' required for loading class '{$className}' doesn't exist.");
        }

        //echo $filePath . "<br />";

        require_once $filePath;

        $definitionExists = class_exists($className, false)
                            || interface_exists($className, false)
                            || trait_exists($className, false);

        if ($definitionExists)
        {
            return true;
        }
        else
        {
            throw new Cx_Exception("Definiton of '{$className}' wasn't found in file '{$filePath}'.");
        }
    }
}