<?php
/**
 * Action
 *
 * @author Semenov Alexander <semenov@skeeks.com>
 * @link http://skeeks.com/
 * @copyright 2010-2014 SkeekS (Sx)
 * @date 07.10.2014
 * @since 1.0.0
 */

/**
 * Class Cx_Controller_Action
 */
class Cx_Controller_Action
{
    /**
     * @var Cx_Controller_Request
     */
    public $request = null;

    /**
     * @var Cx_Controller_Response|Cx_Controller_Response_Http|Cx_Controller_Response_Http_Json
     */
    public $response = null;

    /**
     * @var Cx_View
     */
    public $view = null;

    /**
     * @var null
     */
    public $id = null;

    public $action = null;

    /**
     * @param null $id
     */
    public function __construct($id = null)
    {
        $this->id = $id;
        $this->_init();
        $this->_validate();
    }


    protected function _validate()
    {}

    /**
     * Extensions of the constructor
     * @return void
     */
    protected function _init()
    {
        $this->view            = new Cx_View();
    }


    /**
     * @param Cx_Controller_Request $request
     * @param Cx_Controller_Response $response
     * @throws Cx_Exception
     */
    public function dispatch(Cx_Controller_Request $request, Cx_Controller_Response $response)
    {
        $this->request  = $request;
        $this->response = $response;

        $this->_preDispatch();
        $this->_dispatchAction($request->getActionName());
        $this->_postDispatch();
    }


    /**
     * @return void
     */
    protected function _preDispatch()
    {}

    /**
     * @return void
     */
    protected function _postDispatch()
    {}

    /**
     * @param $actionName
     * @throws Cx_Exception
     */
    protected function _dispatchAction($actionName)
    {
        $this->action = $actionName;

        $actionMethod   = 'action' . ucfirst($actionName);

        if (!method_exists($this, $actionMethod))
        {
            throw new Cx_Exception("Action '{$actionName}' wasn't found in controller - " . get_called_class(), 404);
        }

        //set the dispatched flag
        $this->request->setDispatched(true);

        //call the action
        $this->{$actionMethod}();
    }
}