<?php
/**
 * Request
 *
 * @author Semenov Alexander <semenov@skeeks.com>
 * @link http://skeeks.com/
 * @copyright 2010-2014 SkeekS (Sx)
 * @date 20.02.2015
 * @since 1.0.0
 */
class Cx_Controller_Request
{
    CONST OPTION_LANGUGE           = "language";
    CONST OPTION_REAL_HOSTNAME     = "host";

    CONST OPTION_RESPONSE          = "response";
    CONST OPTION_REFERER           = "referer";

    /**
     * @var Cx_Namespace|null
     */
    protected $_namespace = null;

    /**
     * @var string|null
     */
    protected $_componentName = null;
    /**
     * @var string|null
     */
    protected $_controllerName = null;
    /**
     * @var string|null
     */
    protected $_actionName = null;



    /**
     * @var array Параметры запросы
     */
    protected $_params = array();

    //Также в запросе могут прийди или указание локали или указание языка
    /**
     * @var array
     */
    protected $_options = [];



    /**
     * @var bool Was this request routed?
     */
    protected $_isRouted = false;

    /**
     * @var Cx_Controller_Router_Route
     */
    protected $_usedRoute = null;

    /**
     * @var bool Was this request dispatched?
     */
    protected $_isDispatched = false;

    /**
     * @var Exception|null The exception raised during execution
     */
    protected $_exception = null;


    public function route($controllerName, $actionName, array $params = [], array $options = [])
    {
        $this->_controllerName = $controllerName;
        $this->_actionName     = $actionName;
        $this->_params = $params;

        $this->_options = $options;

        //set routed
        $this->_isRouted = true;

        return $this;
    }

    /**
     * @param array $options
     * @return $this
     */
    public function setOptions($options = [])
    {
        $this->_options = $options;
        return $this;
    }

    /**
     * @return bool
     */
    public function isRouted()
    {
        return $this->_isRouted;
    }

    /**
     * @param  Cx_Controller_Router_Route $route
     * @return Cx_Controller_Request
     */
    public function setUsedRoute(Cx_Controller_Router_Route $route)
    {
        $this->_usedRoute = $route;
        return $this;
    }

    /**
     * @return Cx_Controller_Router_Route|null
     */
    public function getUsedRoute()
    {
        return $this->_usedRoute;
    }

    /**
     * @return bool
     */
    public function isDispatched()
    {
        return $this->_isDispatched;
    }

    /**
     * @param  bool $flag
     * @return Cx_Controller_Request
     */
    public function setDispatched($flag)
    {
        $this->_isDispatched = (bool)$flag;
        return $this;
    }

    /**
     * @return null|string
     */
    public function getActionName()
    {
        return $this->_actionName;
    }

    /**
     * @return null|string
     */
    public function getControllerName()
    {
        return $this->_controllerName;
    }

    /**
     * @return Cx_Namespace
     */
    public function getNamespace()
    {
        return $this->_namespace;
    }


    /**
     * @return null|string
     */
    public function getComponentName()
    {
        return $this->_componentName;
    }



    /**
     * @param $value
     * @return $this
     */
    public function setComponentName($value)
    {
        $this->_componentName = $value;
        return $this;
    }

    /**
     * @param $value
     * @return $this
     */
    public function setControllerName($value)
    {
        $this->_controllerName = $value;
        return $this;
    }

    /**
     * @param $value
     * @return $this
     */
    public function setActionName($value)
    {
        $this->_actionName = $value;
        return $this;
    }


    /**
     * @return bool
     */
    public function hasException()
    {
        return (bool)$this->_exception;
    }

    public function getException()
    {
        return $this->_exception;
    }

    /**
     * @param Exception $exception
     * @return Cx_Controller_Request
     */
    public function setException(Exception $exception)
    {
        $this->_exception = $exception;
        return $this;
    }

    /**
     * @return array
     */
    public function getOptions()
    {
        return $this->_options;
    }

    /**
     * @param null $key
     * @param null $default
     * @return array|mixed|null
     */
    public function getOption($key = null, $default = null)
    {
        return Cx_Array::getKeyOrArray($this->_options, $key, $default);
    }


    /**
     * @param null $key
     * @param null $default
     * @return array|null
     */
    public function getParam($key = null, $default = null)
    {
        return Cx_Array::getKeyOrArray($this->_params, $key, $default);
    }

    /**
     * @return array
     */
    public function getParams()
    {
        return $this->_params;
    }


    /**
     * @param  string $type
     * @param  string $name
     * @throws Cx_Controller_Exception
     */
    protected function _checkName($type, $name)
    {
        if (!Cx_Validate::isValid(new Cx_Validator_Is_McaName(), $name))
        {
            throw new Cx_Controller_Exception("'{$name}' is not a valid {$type} name.'", 404);
        }
    }





    //TODO::добавить еще локаль

    /**
     * @return string
     */
    public function getOptionLanguage()
    {
        return (string) $this->getOption(self::OPTION_LANGUGE);
    }

    /**
     * @return string
     */
    public function getOptionHost()
    {
        return (string) $this->getOption(self::OPTION_REAL_HOSTNAME);
    }



    protected $_host = null;
    /**
     *
     * Вренет реальный хости или из опции (router собирет запрос, выделит реальный хост в опцию)
     * Вренет httpHost из массива сервер, если не будет перопределно опции.
     *
     * @return Cx_Url_Host
     */
    public function getHost()
    {
        if ($this->_host === null)
        {
            if ($host = $this->getOptionHost())
            {
                $this->_host = new Cx_Url_Host($host);
            } else
            {
                $this->_host = new Cx_Url_Host($this->getServer()->getHttpHost());
            }
        }

        return $this->_host;
    }



    protected $_homeUrl = null;

    /**
     * @return Cx_Url
     * @throws Cx_ExceptionSxInvalidArgument
     */
    public function getHomeUrl()
    {
        if ($this->_homeUrl === null)
        {
            $this->_homeUrl = \Cx_Url::create($this->getHost())->setScheme($this->getServer()->getScheme());
        }

        return $this->_homeUrl;
    }


    /**
     * @return Cx_Controller_Request_Server
     */
    public function getServer()
    {
        return Cx_Controller_Request_Server::getInstance();
    }


    /**
     * @return string
     */
    public function getScheme()
    {
        return $this->getServer()->getScheme();
    }








    /**
     * @return string
     */
    public function getRequestUri()
    {
        return $this->getServer()->getRequestUri();
    }

    /**
     * @return string
     */
    public function getRedirectUrl()
    {
        return  $this->getServer()->getRedirectUrl();
    }





    protected $_url = null;

    /**
     * @return Cx_Url
     */
    public function getUrl()
    {
        if ($this->_url === null)
        {
            $url = Cx_Url::getInstance($this->getRequestUri());
            $url->setScheme($this->getServer()->getScheme());
            $url->setHost($this->getServer()->getHttpHost());
            $this->_url = $url;
        }

        return $this->_url;
    }


    /**
     * @param null $default
     * @return string
     */
    public function getOptionResponse($default = null)
    {
        return (string) $this->getOption(self::OPTION_RESPONSE);
    }

    /**
     * @param null $default
     * @return string
     */
    public function getOptionReferer($default = null)
    {
        return (string) $this->getOption(self::OPTION_REFERER, $default);
    }


    /**
     * @return Cx_Controller_Response|Cx_Controller_Response_Http_Html|Cx_Controller_Response_Http_Json
     */
    public function createDefaultResponse()
    {
        //Если в опциях реквеста содержится указание response, тогда по умолчаню попытаемся ответить как надо
        if ($response = $this->getOptionResponse())
        {
            $filter = new Cx_Filter_String_DashToCamelCase();
            $classResponse = "Cx_Controller_Response_Http_" . $filter->filter($response);
            return new $classResponse();
        }

        //Если это ajax то отвечаем в json по умолчанию
        if ($this->getServer()->isAjax())
        {
            return new Cx_Controller_Response_Http_Json();
        } else
        {
            return new Cx_Controller_Response_Http_Html();
        }
    }


    /**
     * @return Cx_Controller_Request_Post
     */
    public function getPost()
    {
        return Cx_Controller_Request_Post::getInstance();
    }

    /**
     * @return Cx_Controller_Request_Query
     */
    public function getQuery()
    {
        return Cx_Controller_Request_Query::getInstance();
    }

    /**
     * @return Cx_Controller_Request_Files
     */
    public function getFiles()
    {
        return Cx_Controller_Request_Files::getInstance();
    }


    /**
     * @return Cx_Controller_Request_Cookie
     */
    public function getCookie()
    {
        return Cx_Controller_Request_Cookie::getInstance();
    }


    /**
     * @return bool
     */
    public function isAjax()
    {
        return $this->getServer()->isAjax();
    }


    /**
     * @return bool
     */
    public function isFiles()
    {
        return (bool) $this->getFiles()->toArray();
    }

    /**
     * @return bool
     */
    public function isPost()
    {
        return (bool) $this->getServer()->isPost();
    }

    /**
     * @return bool
     */
    public function isGet()
    {
        return (bool) $this->getServer()->isGet();
    }

}