<?php
/**
 * Server
 *
 * @author Semenov Alexander <semenov@skeeks.com>
 * @link http://skeeks.com/
 * @copyright 2010-2014 SkeekS (Sx)
 * @date 13.10.2014
 * @since 1.0.0
 */

/**
 * Class Cx_Controller_Request_Server
 */
class Cx_Controller_Request_Server
{
    use Tx_Singleton;
    use Tx_Entity;

    protected function __construct()
    {
        $this->_data = static::getOriginalData();
    }

    /**
     * @return array
     */
    static public function getOriginalData()
    {
        return (array) $_SERVER;
    }




    /**
     * @return bool
     */
    public function isAjax()
    {
        return (bool) ($this->get("HTTP_X_REQUESTED_WITH") == 'XMLHttpRequest');
    }

    /**
     * @return bool
     */
    public function isPost()
    {
        return (bool) ($this->get("REQUEST_METHOD") == 'POST');
    }

    /**
     * @return bool
     */
    public function isGet()
    {
        return (bool) ($this->get("REQUEST_METHOD") == 'GET');
    }


    /**
     * @return string
     */
    public function getRequestUri()
    {
        return (string) $this->get("REQUEST_URI");
    }

    /**
     * @return string
     */
    public function getRedirectUrl()
    {
        return (string) $this->get("REDIRECT_URL");
    }

    /**
     * @return string
     */
    public function hasUrlReferer()
    {
        return (bool) $this->get("HTTP_REFERER");
    }

    /**
     * @return string
     */
    public function getUrlReferer()
    {
        return (string) $this->get("HTTP_REFERER");
    }


    /**
     * Проверяет дополнительный заголовок
     * HTTP_X_REAL_HOST (можно настроить в nginx)
     *
     * @return string
     */
    public function getHttpHost()
    {
        if ($this->get("HTTP_X_REAL_HOST"))
        {
            return (string) $this->get("HTTP_X_REAL_HOST");
        } else if ($this->get("HTTP_HOST"))
        {
            return (string) $this->get("HTTP_HOST");
        } else
        {
            return (string) $this->get("SERVER_NAME");
        }
    }

    /**
     * @return string
     */
    public function getScheme()
    {
        if ($this->get("HTTPS") == "on" || $this->get("HTTPS") == 1 || $this->get("HTTP_X_FORWARDED_PROTO") == "https")
        {
            return Cx_Url::SCHEME_HTTPS;
        }

        return Cx_Url::SCHEME_HTTP;
    }


    /**
     * @return string
     */
    public function getUserIp()
    {
        if ($this->get("HTTP_X_FORWARDED_FOR"))
        {
    	    $ips = explode(', ', $this->get("HTTP_X_FORWARDED_FOR"));
    	    return $ips[0];
        }

        if ($this->get("HTTP_X_REAL_IP"))
        {
            return $this->get("HTTP_X_REAL_IP");
        }

        return $this->getUserHostAddress();
    }






    /**
	 * Returns the server name.
	 * @return string server name
	 */
	public function getServerName()
	{
		return (string) $this->get("SERVER_NAME");
	}

	/**
	 * Returns the server port number.
	 * @return integer server port number
	 */
	public function getServerPort()
	{
		return (string) $this->get("SERVER_PORT");
	}

	/**
	 * Returns the user agent, null if not present.
	 * @return string user agent, null if not present
	 */
	public function getUserAgent()
	{
		return (string) $this->get("HTTP_USER_AGENT");
	}

	/**
	 * Returns the user IP address.
	 * @return string user IP address
	 */
	public function getUserHostAddress()
	{
		return (string) $this->get("REMOTE_ADDR", "127.0.0.1");
	}

	/**
	 * Returns the user host name, null if it cannot be determined.
	 * @return string user host name, null if cannot be determined
	 */
	public function getUserHost()
	{
		return isset($_SERVER['REMOTE_HOST'])?$_SERVER['REMOTE_HOST']:null;
	}

    /**
	 * Returns user browser accept types, null if not present.
	 * @return string user browser accept types, null if not present
	 */
	public function getAcceptTypes()
	{
		return (string) $this->get("HTTP_ACCEPT");
	}
}