<?php
/**
 * Response
 *
 * @author Semenov Alexander <semenov@skeeks.com>
 * @link http://skeeks.com/
 * @copyright 2010-2014 SkeekS (Sx)
 * @date 07.10.2014
 * @since 1.0.0
 */

interface Ix_Controller_Response
{
    /**
     * @param array $headers
     * @return $this
     */
    public function setHeaders($headers);

    /**
     * @return array
     */
    public function getHeaders();

    /**
     * @return mixed
     */
    public function getBody();

    /**
     * @param string $body
     * @return mixed
     */
    public function setBody($body);

    /**
     * Произвести ответ
     * @return mixed
     */
    public function execute();

}

/**
 * Class Cx_Controller_Response
 */
abstract class Cx_Controller_Response
    extends Cx_Entity
    implements Ix_Controller_Response
{

    /**
    * @var array Set HTTP headers
    */
    protected $_headers = [];


    /**
    * @var array The body of the response (chunks)
    */
    protected $_body = [];


    /**
    * @return string
    */
    public function getBody()
    {
        return join('', $this->_body);
    }

    /**
    * @param  string $body
    * @return $this
    */
    public function setBody($body)
    {
        $this->_body = array((string) $body);
        return $this;
    }

    /**
    * @param  string $content
    * @return Cx_Controller_Response
    */
    public function appendBody($content)
    {
        $this->_body[] = (string) $content;
        return $this;
    }




    /**
     * @param string $header
     * @param string $content
     * @return $this
     */
    public function setHeaderNameContent($header, $content)
    {
        $header  = $this->_normalizeHeader($header);
        $content = (string) $content;

        $this->addHeaders($header . ': ' . $content);
        return $this;
    }

    /**
     * Добавить зголовки ответа
     * @param string|array $headers
     * @return $this
     */
    public function addHeaders($headers)
    {
        if (is_string($headers))
        {
            $this->_headers[] = $headers;
        } else if (is_array($headers))
        {
            $this->_headers = array_merge($this->_headers, $headers);
        }

        return $this;
    }

    /**
     * Установить заголовки ответа.
     * @param array $headers
     * @return $this
     */
    public function setHeaders($headers = [])
    {
        $this->_headers = $headers;
        return $this;
    }

    /**
    * Normalize a header name
    *
    * Normalizes a header name to X-Capitalized-Names
    *
    * @param  string $name
    * @return string
    */
    protected function _normalizeHeader($name)
    {
        $filtered = str_replace(['-', '_'], ' ', (string) $name);
        $filtered = ucwords(strtolower($filtered));
        $filtered = str_replace(' ', '-', $filtered);
        return $filtered;
    }

    /**
    * @return array
    */
    public function getHeaders()
    {
        return $this->_headers;
    }


    /**
     * @return $this
     */
    protected function _addDefaultHeaders()
    {
        $this->setHeaderNameContent("x-powered-by-framework", 'SkeekS Cms Installer');
        return $this;
    }

    /**
     * @return $this
     */
    protected function _executeHeaders()
    {
        if ($this->getHeaders())
        {
            foreach ($this->getHeaders() as $headerValue)
            {
                header($headerValue);
            }
        }

        return $this;
    }


    /**
     * @return $this
     */
    public function execute()
    {
        $this
            ->_addDefaultHeaders()
            ->_executeHeaders()
            ->_execute()
        ;

        return $this;
    }

    abstract protected function _execute();
}