<?php
/**
 * Response
 *
 * @author Semenov Alexander <semenov@skeeks.com>
 * @link http://skeeks.com/
 * @copyright 2010-2014 SkeekS (Sx)
 * @date 07.10.2014
 * @since 1.0.0
 */


/**
 * Class Cx_Controller_Response_Http_Json
 */
class Cx_Controller_Response_Http_Json
    extends Cx_Controller_Response_Http
{

    public function init()
    {
        parent::init();

        $this->setDefaultData([
            "success"   => true,
            "errors"    => [],
        ]);

        /*$this->setDefaultOptions([
            "enableBody" => false
        ]);*/
    }

    /**
     * @return $this
     */
    protected function _addDefaultHeaders()
    {
        parent::_addDefaultHeaders();

        $this->setHeaderNameContent("Content-Type", "text/json; charset=utf-8");
        return $this;
    }

    /**
     * @return $this
     */
    protected function _execute()
    {
        /*if ($this->isEnbaleBody())
        {
            $this->addData([
                'debug-sx'      => Cx_Debug::finishInArray()
            ]);
        }

        $this->addData([
            'debug-sx'      => Cx_Debug::finishInArray()
        ]);*/

        echo $this->toJson();
        return $this;
    }



        /**
         * Полезные стандартные опции
         */

    /**
     * Включить body в json response
     * @return $this
     */
    public function enableBody()
    {
        return $this->setOption("enableBody", true);
    }

    /**
     * Не включать body в json response
     * @return $this
     */
    public function disableBody()
    {
        return $this->setOption("enableBody", false);
    }

    /**
     * @return bool
     */
    public function isEnbaleBody()
    {
        return (bool) $this->get("enableBody", false);
    }


        /**
         * Полезные стандартные функции для изменения значений data
         */



    /**
     * @param bool $value
     * @return $this
     */
    public function setSuccessful($value = true)
    {
        return $this->set("success", $value);
    }

    /**
     * @param string|\Cx_Url $redirectUrl
     * @return $this
     */
    public function setRedirect($redirectUrl)
    {
        return $this->set("redirect", (string) $redirectUrl);
    }


    /**
     * @param array $errors
     * @return $this
     */
    public function setErrors(array $errors)
    {
        return $this->setSuccessful(false)->set("errors", $errors);
    }

    /**
     * @param string $error
     * @return $this
     */
    public function addError($error)
    {
        $this->setSuccessful(false);
        $this->_data["errors"][] = (string) $error;
        return $this;
    }

    /**
     * @param Exception $e
     * @return $this
     */
    public function setException(Exception $e)
    {
        if ($e instanceof Cx_ExceptionSxRedirect)
        {
            $this->setSuccessful(true)->setRedirect($e->getMessage());
        } else
        {
            $this->setSuccessful(false)->addError($e->getMessage());
        }

        $this->_data["exception"] =  [
            "message"   => $e->getMessage(),
            "file"      => $e->getFile(),
            "line"      => $e->getLine(),
            "trace"     => $e->getTrace(),
        ];

        return $this;
    }

    /**
     * @param array $info
     * @return $this
     */
    public function addPhpError($info = [])
    {
        if (isset($info["isFatal"]))
        {
            if ($info["isFatal"] == 1)
            {
                $this
                    ->setSuccessful(false)
                    ->addError("Непредвиденная ошибка, обратитесь к разработчику")
                ;
            }
        }

        $this->_data["php-errors"][] = $info;

        return $this;
    }
}