<?php
/**
 * class Application
 *
 * Обложка для доступа к приложению + bootstraper
 *
 * @author Ayupov Vladimir
 * @since 22.09.2010
 */
class SkeeksBase
{
    /**
     * @var Cx_Application|null Current application instance
     */
    static public $app = null;

    /**
     * @var Cx_ClassAutoLoader|null
     */
    static protected $_autoloader = null;

    /**
     * @static
     * @param array|null $config
     * @return Cx_Application_Web
     */
    static public function createWebApplication(array $config = [])
    {
        return static::_createApplication('Cx_Application_Web', $config);
    }

    /**
     * @static
     * @param  array|null $config
     * @return Cx_Application_Cli
     */
    static public function createCliApplication(array $config = [])
    {
        return static::_createApplication('Cx_Application_Cli', $config);
    }

    /**
     * @param $class
     * @param $config
     * @return Cx_Application
     */
    static public function createApplication($class, array $config = [])
    {
        return static::_createApplication($class, $config);
    }


    /**
     * @param $class
     * @param $config
     * @return Cx_Application|null
     * @throws Cx_Exception
     */
    static protected function _createApplication($class, $config = [])
    {
        if (null !== static::$app)
        {
            throw new Cx_Exception("Application can only be created once.");
        }

        static::$app   = new $class($config);
        return static::$app;
    }

    /**
     * @static
     * @return Cx_ClassAutoLoader
     */
    static public function getAutoloader()
    {
        //use framework classes loader
        if (null === static::$_autoloader)
        {
            static::$_autoloader = new Cx_ClassAutoloader(
                [APP_DIR, SX_DIR]
            );
        }

        return static::$_autoloader;
    }
}


// Registering default Skeeks class autoloader
spl_autoload_register([SkeeksBase::getAutoloader(), 'loadClass']);