<?php
/**
 * Uri
 *
 * @date 21.03.14
 * @copyright skeeks.com
 * @author Semenov Alexander <semenov@skeeks.com>
 */

/**
 * Class Cx_Url
 */
class Cx_Url
    extends Cx_Entity
{
    const SCHEME_MULTI  = "";
    const SCHEME_HTTP   = "http";
    const SCHEME_HTTPS  = "https";

    /**
     * @param $url
     * @throws Cx_Exception
     */
    static public function create($url)
    {
        if ($url instanceof Cx_Url)
        {
            $new = new static();
            $new->setData($url->toArray());
            return new static($new);

        } else if ($url instanceof Cx_Url_Host)
        {
            $urlNew = new static();
            $urlNew->setHost($url);
            return $urlNew;
        } else
        {
            if ($url)
            {
                return new static($url);
            } else
            {
                throw new Cx_Exception("Url must be string or \\Cx_Url");
            }
        }
    }

    /**
     *
     * Если передан объект Cx_Url то просто вернет его
     * Если нет то пересоберет
     *
     * @param $url
     * @return Cx_Url
     */
    static public function getInstance($url)
    {
        if ($url instanceof Cx_Url)
        {
            return $url;
        } else
        {
            return static::create($url);
        }
    }


    /**
     * @param string $url
     */
    public function __construct($url = "")
    {
        if ($url)
        {
            $this->_parse((string) $url);
        }

        $this->_init();
    }

    protected function _init()
    {}

    /**
     * @param $url
     * @return $this
     */
    protected function _parse($url = "")
    {
        $this->_data            = parse_url($url);

        if ($this->offsetGet("host"))
        {
            $this->setHost($this->_data["host"]);
        }

        if ($this->isQuery())
        {
            $this->setQuery($this->_data["query"]);
        }

        return $this;
    }


    /**
     * @return string
     */
    public function toString()
    {
        $paths = [];

        if ($this->getHost())
        {
            $paths[] = $this->getScheme() ? $this->getScheme() . ":" : "";
            $paths[] = '//';

            $paths[] = $this->getUser() ? $this->getUser() : "";
            $paths[] = $this->getPass() ? ":" . $this->getPass() : "";

            $paths[] = $this->getUser() || $this->getPass() ? '@' : '';

            $paths[] = $this->getHost()->toString();

            $paths[] = $this->getPort() ? ":" . $this->getPort() : "";
        }


        $paths[] = $this->getPath() ? $this->getPath() : '';
        $paths[] = !$this->getQuery()->isEmpty() ? "?" . (string) $this->getQuery() : '';
        $paths[] = $this->getFragment() ? "#" . (string) $this->getFragment() : '';

        return implode($paths);
    }



    /**
     * @return string
     */
    public function __toString()
    {
        return $this->toString();
    }


    /**
     * Есть query в строке
     * @return bool
     */
    public function isQuery()
    {
        return (bool) $this->offsetGet("query");
    }



    /**
     * @return string
     */
    public function getPass()
    {
        return (string) $this->get("pass");
    }


    /**
     * @return string
     */
    public function getScheme()
    {
        return (string) $this->get("scheme", static::SCHEME_HTTP);
    }

    /**
     * @return Cx_Url_Host
     */
    public function getHost()
    {
        return $this->get("host");
    }

    /**
     * @return string
     */
    public function getPath()
    {
        return (string) $this->get("path");
    }

    /**
     * @return Cx_Url_Query
     */
    public function getQuery()
    {
        if (!$this->get("query"))
        {
            $this->set("query", Cx_Url_Query::createEntity());
        }

        return $this->get("query");
    }

    /**
     * @return string
     */
    public function getPort()
    {
        return (string) $this->get("port");
    }

    /**
     * @return string
     */
    public function getUser()
    {
        return (string) $this->get("user");
    }



    /**
     * @return string
     */
    public function getFragment()
    {
        return (string) $this->get("fragment");
    }


    /**
     * @param $value
     * @return $this
     */
    public function setScheme($value)
    {
        return $this->set("scheme", $value);
    }

    /**
     * @param string|\Cx_Url_Host $host
     * @return $this
     */
    public function setHost($value)
    {
        if ($value && is_string($value))
        {
            $value = new Cx_Url_Host($value);
        }

        if (!$value instanceof Cx_Url_Host)
        {
            //throw new Cx_ExceptionSxInvalidArgument("host must be string or \\Cx_Url_Host");
        }

        return $this->set("host", $value);
    }

    /**
     * @param $value
     * @return $this
     */
    public function setPath($value)
    {
        return $this->set("path", $value);
    }

    /**
     * @param strintg|array|\Cx_Url_Query $value
     * @return $this
     */
    public function setQuery($value)
    {
        if (is_array($value))
        {
            $value = new Cx_Url_Query($value);
        } else if (Cx_Validate::isValid(new Cx_Validator_Not_EmptyString(), $value))
        {
            $value = Cx_Url_Query::createFromString($value);
        }

        if (!Cx_Validate::isValid(new Cx_Validator_InstanceOf("Cx_Url_Query"), $value))
        {
            throw new Cx_ExceptionSxInvalidArgument("query must be string or \\Cx_Url_Query or array");
        }

        return $this->set("query", $value);
    }

    /**
     * @param $value
     * @return $this
     */
    public function setPort($value)
    {
        return $this->set("port", $value);
    }

    /**
     * @param $value
     * @return $this
     */
    public function setUser($value)
    {
        return $this->set("user", $value);
    }

    /**
     * @param $value
     * @return $this
     * @throws Cx_Exception
     */
    public function setPass($value)
    {
        return $this->set("pass", $value);
    }

    /**
     * @param $value
     * @return $this
     */
    public function setFragment($value)
    {
        return $this->set("fragment", $value);
    }


    /**
     * @return $this
     */
    public function setCurrentHost()
    {
        $this
            ->setHost(Sx::app()->getHomeUrl()->getHost())
            ->setScheme(Sx::app()->getHomeUrl()->getScheme())
        ;

        return $this;
    }

    /**
     * @return Cx_Url_Headers
     */
    public function getHeaders()
    {
        if ($this->get("headers", null) === null)
        {
            $this->set("headers", $this->getHeaders());
        }

        return $this->get("headers");
    }

    /**
     * @return Cx_Url_Headers
     */
    public function fetchHeaders()
    {
        return Cx_Url_Headers::fetchByUrl($this);
    }


    /**
     * @var Cx_Url_Helper
     */
    protected $_helper = null;

    /**
     * @return Cx_Url_Helper
     */
    public function getHelper()
    {
        if ($this->_helper === null)
        {
            $this->_helper = new Cx_Url_Helper($this);
        }

        return $this->_helper;
    }


    /**
     *
     * Убирает лишние слэши из пути
     *
     * @param string $path
     * @return string
     */
    static public function removeSlashes($path = "")
    {
        if ($path)
        {
            $array = explode("/", $path);
            $array = Cx_Helper_Array::onlyNotEmpty($array);
            $path = "/" . implode("/", $array);
        }
        return $path;
    }
}