<?php
/**
 * Uri
 *
 * @date 21.03.14
 * @copyright skeeks.com
 * @author Semenov Alexander <semenov@skeeks.com>
 */

/**
 * Class Cx_Url_Host
 */
class Cx_Url_Host
{
    /**
     * @param $file
     * @return static
     */
    static public function object($file = null)
    {
        if ($file instanceof static)
        {
            return $file;
        } else
        {
            return new static($file);
        }
    }

    /**
     * @param string|Cx_Url $url
     * @return Cx_Url_Host
     * @throws Cx_ExceptionSxInvalidArgument
     */
    static public function create($url)
    {
        if ($url instanceof Cx_Url_Host)
        {
            return new static($url->getHostName());
        } else
        {
            if (Cx_Validate::isValid(new Cx_Validator_Not_EmptyString(), $url))
            {
                return new static($url);
            } else
            {
                throw new Cx_ExceptionSxInvalidArgument("Url must be string or \\Cx_Url");
            }
        }
    }

    /**
     * @param $url
     * @return Cx_Url_Host
     */
    static public function getInstance($url)
    {
        if ($url instanceof Cx_Url_Host)
        {
            return $url;
        } else
        {
            return static::create($url);
        }
    }

    protected $_hostName    = "";

    protected $_www         = false;
    /**
     * @param string $url
     */
    public function __construct($hostName)
    {
        $this->_parse((string) $hostName);
    }

    /**
     * @param $url
     * @return $this
     */
    protected function _parse($hostName)
    {
        if (static::hostNameIsWwww($hostName))
        {
            $this->_www         = true;
            $this->_hostName    = substr($hostName, 4, strlen($hostName));
        } else
        {
            $this->_hostName    = $hostName;
            $this->_www         = false;
        }

        return $this;
    }

    /**
     * @return string
     */
    public function getHostName()
    {
        return (string) $this->_hostName;
    }

    /**
     * @return string
     */
    public function toString()
    {
        $result = [];

        if ($this->isWwww())
        {
            $result[] = "www.";
        }

        $result[] = $this->getHostName();
        return implode($result);
    }

    /**
     * @return Cx_Url
     */
    public function getUrl()
    {
        return Cx_Url::create($this);
    }

    /**
     * @return string
     */
    public function __toString()
    {
        return $this->toString();
    }


    /**
     * @return $this
     */
    public function enableWww()
    {
        $this->_www = true;
        return $this;
    }

    /**
     * @return $this
     */
    public function disableWww()
    {
        $this->_www = false;
        return $this;
    }

    /**
     * @return bool
     */
    public function isWwww()
    {
        return $this->_www;
    }




    /**
     * @return bool
     */
    static public function hostNameIsWwww($hostName)
    {
        if (substr($hostName, 0, 4) == "www.")
        {
            return true;
        }

        return false;
    }

    /**
     * @return int
     */
    public function getLevel()
    {
        return count(explode(".", $this->_hostName));
    }
}