<?php
/**
 * View
 *
 * @author Semenov Alexander <semenov@skeeks.com>
 * @link http://skeeks.com/
 * @copyright 2010-2014 SkeekS (Sx)
 * @date 07.10.2014
 * @since 1.0.0
 */

/**
 * Interface Ix_View
 */
interface Ix_View
{
    public function render($name);

    public function addPath($path);
    public function setPaths(array $paths = array());
    public function getPaths();

    public function setVar($name, $value);
    public function setVars(array $vars);
    public function getVar($name);
    public function getVars();

    /**
     * @abstract
     * @param  string $name
     * @return Ix_View_Helper
     */
    static public function getHelper($name);
}

/**
 * @TODO: list common view helper here
 * @method Cx_View_Helper_Html_StaticInclude staticInclude()
 * @method Cx_View_Helper_Html html()
 * @method Cx_View_Helper_StaticKey staticKey()
 *
 * Class Cx_View
 */
class Cx_View
{
    use Tx_Entity;

    /**
     * @var array Only admin-wise scripts can actually modify this. Strive to use "absolute" template paths, since they can be used within "dynamic" templates
     */
    protected $_paths = [];

    /**
     * @static
     * @var array
     */
    static protected $_helpers = array();

    /**
     * @param array $config
     */
    public function __construct($config = [])
    {
        if(is_array($config))
        {
            foreach($config as $k => $v)
            {
                if(in_array($k, array('paths', 'vars')))
                {
                    $this->{'_' . $k} = $v;
                }
            }
        }
    }

    /**
     * @param  string $path
     * @return Cx_View
     */
    public function addPath($path)
    {
        $this->_paths[] = $path;
        return $this;
    }

    /**
     * @param  array $paths
     * @return Cx_View
     */
    public function addPaths(array $paths)
    {
        $this->_paths = array_merge($this->_paths, $paths);
        return $this;
    }

    /**
     * @param array $paths
     * @return Cx_View
     */
    public function setPaths(array $paths = array())
    {
        $this->_paths = $paths;
        return $this;
    }

    /**
     * @return array
     */
    public function getPaths()
    {
        return $this->_paths;
    }



    /**
     * @param $name
     * @param array $vars
     * @return string
     * @throws Cx_Exception
     */
    public function render($name, $vars = array())
    {
        $engineName = $this->_resolveEngineName($name);

        if (!$engineName)
        {
            throw new Cx_Exception("Couldn't resolve template engine name from '{$name}'.");
        }

        //request the template engine
        $engine = \Sx::$app->getTemplateEngine($engineName);
        $this->_data = array_merge((array) $vars, (array) $this->_data);

        //render!
        $result = $engine->renderTemplateFile($this->_paths, $name, (array) $this->toArray());

        return $result;
    }


    /**
     * @param  string $name
     * @return string|null
     */
    protected function _resolveEngineName($name)
    {
        $basename = basename($name);
        if(strpos($name, '.') === false)
        {
            return null;
        }

        $chunks = explode('.', $basename);

        return array_pop($chunks);
    }





    /**
     *
     * TODO: old function, don't use it;
     *
     * @param $name
     * @param $value
     * @return $this
     */
    public function setVar($name, $value)
    {
        return $this->set($name, $value);
    }

    /**
     *
     * TODO: old function, don't use it;
     *
     * @param array $vars
     * @return $this
     */
    public function setVars(array $vars)
    {
        return $this->setData($vars);
    }

    /**
     *
     * TODO: old function, don't use it;
     *
     * @param $name
     * @return null
     */
    public function getVar($name)
    {
        return $this->get($name);
    }

    /**
     * TODO: old function, don't use it;
     * @return array
     */
    public function getVars()
    {
        return $this->toArray();
    }

    /**
     * TODO: old function, don't use it;
     *
     * @param array $vars
     * @return $this
     */
    public function mergeVars(array $vars)
    {
        return $this->addData($vars);
    }





    /**
     * @static
     * @param  string $name
     * @return Ix_View_Helper
     */
    static public function getHelper($name)
    {
        if(!isset(static::$_helpers[$name]))
        {
            static::$_helpers[$name] = static::_createHelper($name);
        }

        $helper = static::$_helpers[$name];
        return $helper;
    }

    /**
     * @static
     * @param  string $name
     * @return Ix_View_Helper
     * @throws Cx_Exception
     */
    static protected function _createHelper($name)
    {
        $class = \Sx::getClassName('Cx_View_Helper_' . $name);
        if (!class_exists($class, true))
        {
            throw new Cx_Exception("View helper '{$name}' doesn't exist.");
        }

        return new $class;
    }

    /**
     * @param string $name
     * @param array  $args
     * @return mixed
     */
    public function __call($name, $args)
    {
        $helper = static::getHelper($name);
        return call_user_func_array($helper, $args);
    }
}