<?php
/**
 * Phtml
 *
 * @author Semenov Alexander <semenov@skeeks.com>
 * @link http://skeeks.com/
 * @copyright 2010-2014 SkeekS (Sx)
 * @date 07.10.2014
 * @since 1.0.0
 */

/**
 * Class Cx_View_TemplateEngine_Phtml
 */
class Cx_View_TemplateEngine_Phtml
    extends Cx_View_TemplateEngine
{
    use Tx_Entity;

    /**
     * @return array
     */
    public function getVars()
    {
        return $this->toArray();
    }

    /**
     * @abstract
     * @param  array  $paths  List of paths
     * @param  string $name   Name of the template file
     * @param  array  $vars   Template context
     * @return string
     * @throws Cx_Exception
     */
    public function renderTemplateFile(array $paths, $name, array $vars = array())
    {
        $templatePath = $this->_resolveTemplatePath($paths, $name);

        if (!$templatePath)
        {
            throw new Cx_Exception("Couldn't find template '{$name}'. Used paths: " . join(', ', $paths) . '.');
        }

        if(!$this->_isSecurePath($templatePath))
        {
            throw new Cx_Exception("Template '{$templatePath}' is not secure to render.'");
        }

        //render (execute script)
        $this->_data = $vars;
        $output      = $this->_render($templatePath);
        $this->_data = array();

        //return output
        return $output;
    }

    /**
     * @param array $paths
     * @param string $name
     * @return null|string
     */
    protected function _resolveTemplatePath(array $paths, $name)
    {
        foreach($paths as $path)
        {
            $templatePath = $path . '/' . $name;
            if (file_exists($templatePath))
            {
                return $templatePath;
            }
        }

        return null;
    }

    /**
     * @param  string $path
     * @return bool
     */
    protected function _isSecurePath($path)
    {
        //@todo: change it!
        //@todo: write validating code
        return true;
    }

    /**
     * @param  string $path
     * @return string
     */
    protected function _render($path)
    {
        ob_start();
        include $path;
        $content = ob_get_contents();
        ob_end_clean();

        return $content;
    }

    /**
     * @static
     * @param string $name
     * @param array $arguments
     * @return mixed|void
     */
    public function __call($name, $arguments)
    {
        $helper = Cx_View::getHelper($name);
        return call_user_func_array($helper, $arguments);
    }
}
